Attribute VB_Name = "GDIp"
Option Explicit

' GDI+ constants:
Private Const QualityModeInvalid As Long = -1&
Private Const QualityModeDefault As Long = 0&
Private Const QualityModeLow As Long = 1&
Private Const QualityModeHigh As Long = 2&

' GDI+ types:
Private Type GdiplusStartupInput
    GdiplusVersion As Long
    DebugEventCallback As Long
    SuppressBackgroundThread As Long
    SuppressExternalCodecs As Long
End Type

' Global:
Private Declare Function GdiplusStartup Lib "gdiplus.dll" (ByRef token As Long, ByRef inputX As GdiplusStartupInput, ByVal Output As Long) As Status
Private Declare Sub GdiplusShutdown Lib "gdiplus.dll" (ByVal token As Long)

' Graphics:
Private Declare Function GdipCreateFromHDC Lib "gdiplus.dll" (ByVal hdc As Long, ByRef Graphics As Long) As Status
Private Declare Function GdipDeleteGraphics Lib "gdiplus.dll" (ByVal Graphics As Long) As Status
Private Declare Function GdipDrawArc Lib "gdiplus.dll" (ByVal Graphics As Long, ByVal Pen As Long, ByVal X As Single, ByVal Y As Single, ByVal nWidth As Single, ByVal nHeight As Single, ByVal startAngle As Single, ByVal sweepAngle As Single) As Status 'https://docs.microsoft.com/de-de/dotnet/api/system.drawing.graphics.drawarc?view=dotnet-plat-ext-5.0
Private Declare Function GdipSetSmoothingMode Lib "gdiplus.dll" (ByVal Graphics As Long, ByVal SmoothingMode As SmoothingMode) As Status
    
' Pens:
Private Declare Function GdipCreatePen1 Lib "gdiplus.dll" (ByVal Color As Long, ByVal Width As Single, ByVal Unit As Unit, ByRef Pen As Long) As Status
Private Declare Function GdipDeletePen Lib "gdiplus.dll" (ByVal Pen As Long) As Status

Public Declare Function GdipDrawLineI Lib "gdiplus.dll" (ByVal Graphics As Long, ByVal Pen As Long, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Status

' GDI+ enums:
Public Enum Status
    OK = 0
    GenericError = 1
    InvalidParameter = 2
    OutOfMemory = 3
    ObjectBusy = 4
    InsufficientBuffer = 5
    NotImplemented = 6
    Win32Error = 7
    WrongState = 8
    Aborted = 9
    FileNotFound = 10
    ValueOverflow = 11
    AccessDenied = 12
    UnknownImageFormat = 13
    FontFamilyNotFound = 14
    FontStyleNotFound = 15
    NotTrueTypeFont = 16
    UnsupportedGdiplusVersion = 17
    GdiplusNotInitialized = 18
    PropertyNotFound = 19
    PropertyNotSupported = 20
    ProfileNotFound = 21
End Enum

Private Enum Unit
    UnitWorld = 0
    UnitDisplay = 1
    UnitPixel = 2
    UnitPoint = 3
    UnitInch = 4
    UnitDocument = 5
    UnitMillimeter = 6
End Enum

Private Enum SmoothingMode
    SmoothingModeInvalid = QualityModeInvalid
    SmoothingModeDefault = QualityModeDefault
    SmoothingModeHighSpeed = QualityModeLow
    SmoothingModeHighQuality = QualityModeHigh
    SmoothingModeNone = QualityModeHigh + 1
    SmoothingModeAntiAlias8x4 = QualityModeHigh + 2
    SmoothingModeAntiAlias = SmoothingModeAntiAlias8x4
    SmoothingModeAntiAlias8x8 = QualityModeHigh + 3
End Enum

Private mInstance As Long
Public mGraphics As Long

Public Function myRGB(ByVal Red As Byte, ByVal Green As Byte, ByVal Blue As Byte)

myRGB = RGB(Blue, Green, Red)

End Function

Public Sub SplitRGB(ByVal lColor As Long, ByRef lRed As Long, ByRef lGreen As Long, ByRef lBlue As Long)

lRed = lColor And &HFF
lGreen = (lColor And &HFF00&) \ &H100&
lBlue = (lColor And &HFF0000) \ &H10000

End Sub

Public Sub StartGDIp()

Dim Data As GdiplusStartupInput

Data.GdiplusVersion = 1

If GdiplusStartup(mInstance, Data, 0) Then
    MsgBox "GDI+ could not be initialized.", vbCritical
    Exit Sub
End If

End Sub

Public Sub CloseGDIp()

If mGraphics Then If GdipDeleteGraphics(mGraphics) Then MsgBox "Graphics object could not be deleted.", vbCritical
Call GdiplusShutdown(mInstance)

End Sub

Public Sub UpdateGDIpObject()

If GdipCreateFromHDC(Form1.Picture1.hdc, mGraphics) Then
    MsgBox "Graphics object could not be created.", vbCritical
    Exit Sub
End If

End Sub

Public Function SetAA() As Boolean

If GdipSetSmoothingMode(mGraphics, SmoothingModeAntiAlias8x8) Then
    MsgBox "Antialiasing mode could not be initialized.", vbCritical
    Exit Function
End If

SetAA = True

End Function

Public Sub DrawLine(X1 As Long, Y1 As Long, X2 As Long, Y2 As Long, Color As Long)

Dim Red As Long
Dim Green As Long
Dim Blue As Long
Dim RGB As Long

Dim MyPen As Long
Dim MyStatus As Status

Call SplitRGB(Color, Red, Green, Blue)
Color = myRGB(Red, Green, Blue)
Color = &HFF000000 + Color

Call UpdateGDIpObject
Call SetAA

MyStatus = GdipCreatePen1(Color, Form1.HScroll2.Value, UnitPixel, MyPen)
    MyStatus = GdipDrawLineI(mGraphics, MyPen, X1, Y1, X2, Y2)
MyStatus = GdipDeletePen(MyPen)

End Sub

